function [expression] = GetExpression(node, pathway, KineticMatrix, expression)

%--------------------------------------------------------------------------
% function used to get the expression given an enzyme form, pathways, and
% KineticMatrix
%
% USAGE: 
%    [expression] = GetExpression(node, pathway, KineticMatrix, expression)
% 
% INPUT:   
%               node  -- Initial node index 
%            pathway  -- Specified a pathway corresponding a K-A pattern
%      KineticMatrix  -- System KineticMatrix, including the rate constants
%                        information
%         expression  -- Equation expression
% 
% OUTPUT:  
%         expression  -- Output expression
%
% also see: KAPattern.m
%---2008--MCW--------------------------------------------------------------
%-- Ver. 1.2 -- F. Qi --

% first step, use the SingleNode to process the first node.
[expression pathway nodes] = SingleNode(node, pathway, KineticMatrix, expression);

node_new = nodes;

while (~isempty(pathway))
    nodes_old = node_new;
    node_new = [];
    for i = 1 : length(nodes_old)
        node = nodes_old(i);
        [expression pathway nodes] = SingleNode(node,pathway,KineticMatrix,expression);
        if(isempty(pathway))
            break;
        end
        node_new = [node_new, nodes]; %#ok<AGROW>
    end
end

function [expression pathway nodes] = SingleNode(node, pathway, KineticMatrix, expression)
%--------------------------------------------------------------------------
% function used to deal with the single node, find all the links whose end
% points are the input node, update the new node list and rest pathway
%
% Called by GetExpression
%
% INPUT: node -- node need to search if it is end point in the pathway list
%        pathway -- all pathways, all links with begin and end point
%        KineticMatrix -- KineticMatrix include the rate constants corresponding
%                      the links
%        expression -- equation expression
% 
% OUTPUT: expression -- updated expression if find new path
%         pathway    -- updated pathway list after delete the found paty
%         nodes      -- updated end points list
%--------------------------------------------------------------------------

nodes = [];
expression_temp = [];
number = 0;
MarkLine = [];

for i = 1 : size(pathway,1)        
    if (node == pathway(i,1))
        nodes = [nodes,pathway(i,2)];                 %#ok<AGROW>
        number = number + 1;  
        MarkLine(number) = i;                         %#ok<AGROW>
        if(KineticMatrix{pathway(i,2),node}==0)
            expression_temp = [expression_temp; '%']; %#ok<AGROW>
        else
            expression_temp = [expression_temp; ...
                      KineticMatrix(pathway(i,2),node)]; %#ok<AGROW>
        end
    elseif (node == pathway(i,2))
        nodes = [nodes, pathway(i,1)];                %#ok<AGROW>
        number = number + 1;
        MarkLine(number) = i;                         %#ok<AGROW>
        if(KineticMatrix{pathway(i,1),node}==0)
            expression_temp = [expression_temp; '%']; %#ok<AGROW>
        else
            expression_temp = [expression_temp; ...
                      KineticMatrix(pathway(i,1),node)]; %#ok<AGROW>
        end
    end    
end

% check if get the new expression term
if(~isempty(expression_temp))
    % true, check expression if already has term
    if (isempty(expression))  
        % no, add expression_temp(1)
        % expression = strcat(expression,expression_temp(1));
        % Comment: the @cell/strcat function is not efficient       
        if (strfind(char(expression_temp(1)), '+')) % old: expression = [expression char(expression_temp(1))];
            expression = [expression '(' char(expression_temp(1)) ')' ];
        else
            expression = [expression char(expression_temp(1))];
        end
    else
        % yes, add '*' and expression_temp(1)
        % expression = strcat(expression,'*',expression_temp(1));
        % see comment above                
        if (strfind(char(expression_temp(1)), '+')) % old: expression = [expression '*' char(expression_temp(1))];
            expression = [expression '*(' char(expression_temp(1)) ')' ];
        else
            expression = [expression '*' char(expression_temp(1))];
        end
    end
    % if has more than one term
    if length(expression_temp) > 1
        for i = 2 : length(expression_temp)
            % yes, add '*' and others
            % expression = strcat(expression,'*',expression_temp(i));                          
            if (strfind(char(expression_temp(i)), '+')) % old :expression = [expression '*' char(expression_temp(i))]; %#ok<AGROW>
                expression = [expression '*(' char(expression_temp(i)) ')' ]; %#ok<AGROW>
            else
                expression = [expression '*' char(expression_temp(i))]; %#ok<AGROW>
            end
        end
    end
end

MarkLine = sort(MarkLine,'descend');
for i = 1 : length(MarkLine)
    pathway(MarkLine(i),:) = [];
end



