function [State Denominator RateList ConList] = KAPattern(filename, fileflag)
%--------------------------------------------------------------------------
% Main function for KAPattern pacakge.
%   
% Generate the enzyme forms fraction, also with rate constants list, and
% substrate concentrations list.
%
% USAGE:  
%        [state Denominator Rate List ConList] = KAPattern(filename)
%
% INPUT:   
%        filename -- (string), input file which descrips the enzyme system
%        fileflag -- (bool), whether output the MathML file(page)
%                    0(default), no generated page, 1, generate two files
%                    one is MathML file, one is HTML page including MathML
% 
% OUTPUT:    
%           State -- (Cell String Array), state{i}/D is the expression for 
%                    enzymefraction in State i.
%
%     Denominator -- {Cell String}, expression for the Denominator for the input
%                    system
%
%        RateList -- {Cell String}, the rate constants list 
%           
%         ConList -- (Cell String)concentrations shown up in pseudo-first-
%                    order rate constants list
%
% Also see: ReadInput.m, GetLink.m, Wang.m, GetExpression.m
%           
%---2008--MCW--------------------------------------------------------------
%-- Ver. 1.0 -- F. Qi --

tic
%% 1. Read the input file
[Nodes Link KineticMatrix] = ReadInput(filename);
[LinkMatrix LinkList] = GetLink(Link);

%% 2. Process using the Wang algebra
NodeNumber = length(Nodes);
temp = [];

MaxIndex = find(cell2mat(Link(:,1))==max(cell2mat(Link(:,1))));
for i = 1 : NodeNumber
    if i == MaxIndex(1)
        continue;
    else
        temp = Wang(temp, Link{i, 3}');
    end
end

Pattern = temp;       % all the possible pattern list

%% 3. Calculate every pattern expression corresponding to each State.
PatternNumber = length(Pattern);
pathway = [];

for i = 1 : PatternNumber
    pattern = Pattern(i,:)';
    pathway_temp = [];
    for j = 1 : length(pattern)
        pathway_temp = [pathway_temp; LinkList(pattern(j),:)];%#ok<AGROW>
    end
    pathway(:,:,i) = pathway_temp;  %#ok<AGROW>
end

expression = cell(length(Nodes),PatternNumber);

for i = 1 : length(Nodes)
    node = Nodes(i);
    for j = 1 : PatternNumber
        exp = '';
        [exp] = GetExpression(node, pathway(:,:,j), KineticMatrix, exp);
        expression{i,j} = exp;
    end
end

%% 4. Finial, output the fraction and Denominator
State = cell(length(Nodes),1);
Denominator = 'D = ';

for i = 1 : length(Nodes);
    temp = char(expression{i,1});
    if (~isempty(strfind(temp,'%')))
        % State{i} = strcat(State{i},'');
        State{i} = [char(State{i}) ''];
    else
        % State{i} = strcat(State{i},' ',expression{i,1});
        State{i} = [char(State{i}) ' ' char(expression{i, 1})];
    end
    for j = 2 : PatternNumber
        temp = char(expression{i,j});
        if(~isempty(strfind(temp,'%')))
            % State{i} = strcat(State{i},'');
            State{i} = [char(State{i}) ''];
        else
            if(isempty(char(State{i})));
                % State{i} = strcat(State{i},expression{i,j});
                State{i} = [char(State{i}) char(expression{i,j})]; 
            else
                % State{i} = strcat(State{i},'+',expression{i,j});
                State{i} = [char(State{i}) '+' char(expression{i,j})];
            end
        end
    end
end

% Denominator = strcat(Denominator,State{1});
Denominator = [Denominator char(State{1})];

State{1} = [char(State{1}) ';'];

for i = 2: length(Nodes);
    % Denominator = strcat(Denominator,'+',State{i});
    % State{i} = strcat(State{i},';');
    Denominator = [char(Denominator) '+' char(State{i})];
    State{i} = [char(State{i}) ';'];
end

if (length(strfind(Denominator,'+')) == length(Denominator)-3)
   Denominator = '';
end
% Denominator = strcat(Denominator,';');
Denominator = [Denominator ';'];

for i = 1 : length(Nodes);
    % State{i} = strcat('E',num2str(i),' = ',State{i});
    State{i} = ['E' num2str(i) '=' char(State{i})];
end

%% 5. Give the rate constant list and the concentration list 
k = 1;
for i = 1 : length(Nodes)
    for j = 1 : length(Nodes)
        if (LinkMatrix(i,j)~=0 && ~isempty(KineticMatrix{i,j}))     
            temp = KineticMatrix{i, j};
            if(isnumeric(temp))
                RateList(k,1) = KineticMatrix(i,j); %#ok<AGROW>
                k = k + 1;
            else
                RateTemp = regexp(temp, '+', 'split');
                for m = 1 : length(RateTemp)
                    RateList(k,1) = RateTemp(m); %#ok<AGROW>
                    k = k + 1;
                end
            end
        end    
    end
end

ConList = {};
j = 1; k = 1;
for i = 1 : length(RateList)
    if (RateList{i} ~= 0)
        temp = char(RateList{i});
        if(strfind(temp,'*'))
            ConList{k} = temp(strfind(temp,'*')+1:end);  %#ok<AGROW>
            k = k + 1;
            RateList_new{j} = temp(1:strfind(temp,'*')-1); %#ok<AGROW>
            j = j + 1;
        else
            RateList_new{j} = temp; %#ok<AGROW>
            j = j + 1;
        end
    end    
end

RateList = sort(RateList_new);
RateList = unique(RateList);
ConList = unique(ConList);

%%  

time = toc; %#ok<NASGU>

% Output the .m file
BuildFile(State, RateList, ConList);

% Output the XML and HTML file
if(fileflag == 1)
    XMLGenerate(State, Denominator);
    HTMLGenerate(filename, State, LinkMatrix, KineticMatrix);
end

end % whole function

%% function used to generate the XML output file
function [] = XMLGenerate(fraction, denominator)
    %----- write the MATH-ML to output textbox
    fid = fopen('Expression.xml', 'w');
    fprintf(fid,'<?xml version="1.0" encoding="UTF-8"?>\n');
    fprintf(fid,'  <math xmlns="http://www.w3.org/1998/Math/MathML">\n');
    fprintf(fid,'  <mtable displaystyle="false" columnalign="left" class="multiline">\n');

    fprintf(fid,'  <mtr>\n');
    fprintf(fid,'  <mtd>\n');
    fprintf(fid,'  <mi>Numerator</mi>\n');
    fprintf(fid,'  <mo>:</mo>\n');
    fprintf(fid,'  </mtd>\n');
    fprintf(fid,'  </mtr>\n');
 
    for a = 1: length(fraction)
        str = char(fraction{a});
        str = regexprep(str, ';', '');
        fprintf(fid,'  <mtr>\n');
        fprintf(fid,'  <mtd>\n');
            for b = 1: length(str)
                if (str(b) == 'E')
                    strb = regexprep(str(b), 'E', '<mi>E</mi>');
                        fprintf(fid,'%s\n',strb);
                    elseif (str(b) == '=')
                        strb = regexprep(str(b), '=', '<mo>&#x0003D;</mo>');
                        fprintf(fid,'%s\n',strb);
                    elseif (str(b) == 'k')
                        strb = regexprep(str(b), 'k', '<mi>k</mi>');
                        fprintf(fid,'%s\n',strb);
                    elseif (str(b) == '_')
                        strb = regexprep(str(b), '_', '<mo>&#x0005F;</mo>');
                        fprintf(fid,'%s\n',strb);
                    elseif (str(b) == '*')
                        strb = regexprep(str(b), '*', '<mo>&#x000D7;</mo>');
                        fprintf(fid,'%s\n',strb);
                    elseif (str(b) == '+')
                        strb = regexprep(str(b), '+', '<mo>&#x0002B;</mo>');
                        fprintf(fid,'%s\n',strb);
                    else
                        fprintf(fid,'<mn>');
                        fprintf(fid,str(b));
                        fprintf(fid,'</mn>\n');
                end
             end
                fprintf(fid,'  </mtd>\n');
                fprintf(fid,'  </mtr>\n');
     end
           
     fprintf(fid,'  <mtr>\n');
     fprintf(fid,'  <mtd>\n');
     fprintf(fid,'  <mi>Denominator</mi>\n');
     fprintf(fid,'  <mo>:</mo>\n');
     fprintf(fid,'  </mtd>\n');
     fprintf(fid,'  </mtr>\n');
            
     str = char(denominator);
     str = regexprep(str, ';', '');
            
     fprintf(fid,'  <mtr>\n');
     fprintf(fid,'  <mtd>\n');
            
     for c = 1 : length(str)
         if (str(c) == 'D')
             strc = regexprep(str(c), 'E', '<mi>E</mi>');
             fprintf(fid,'%s\n',strc);
         elseif (str(c) == '=')
             strc = regexprep(str(c), '=', '<mo>&#x0003D;</mo>');
             fprintf(fid,'%s\n',strc);
         elseif (str(c) == 'k')
             strc = regexprep(str(c), 'k', '<mi>k</mi>');
             fprintf(fid,'%s\n',strc);
         elseif (str(c) == '_')
             strc = regexprep(str(c), '_', '<mo>&#x0005F;</mo>');
             fprintf(fid,'%s\n',strc);
         elseif (str(c) == '*')
             strc = regexprep(str(c), '*', '<mo>&#x000D7;</mo>');
             fprintf(fid,'%s\n',strc);
         elseif (str(c) == '+')
             strc = regexprep(str(c), '+', '<mo>&#x0002B;</mo>');
             fprintf(fid,'%s\n',strc);
         else
             fprintf(fid,'<mn>');
             fprintf(fid,str(c));
             fprintf(fid,'</mn>\n');
         end
     end
              
     fprintf(fid,'  </mtd>\n');
     fprintf(fid,'  </mtr>\n');
     fprintf(fid,'</mtable>\n');
     fprintf(fid,'</math>\n');
    
     fclose(fid);
            
     %winopen('Expression.xml'); 
end

%% function to output the HTML file with MathML equations
function [] = HTMLGenerate(file, state, LinkMatrix, KineticMatrix)
    fid = fopen('ResultPage.html', 'w');
    fprintf(fid,'<HTML XMLNS:m="http://www.w3.org/1998/Math/MathML"> \n');
    fprintf(fid,'<HEAD> \n');
    fprintf(fid,' <OBJECT \n');
    fprintf(fid,'   ID=MathPlayer \n');
    fprintf(fid,'   CLASSID="clsid:32F66A20-7614-11D4-BD11-00104BD3F987"\n');
    fprintf(fid,' > \n');
    fprintf(fid,' </OBJECT> \n');
    fprintf(fid,'<?IMPORT NAMESPACE="m" IMPLEMENTATION="#MathPlayer" ?> \n');
    fprintf(fid,'<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" /> \n');
    fprintf(fid,'<title>KAPattern Result</title> \n');
    fprintf(fid,'<style type="text/css"> \n');
    fprintf(fid,'<!--\n');
    fprintf(fid,'.style6 {\n');
    fprintf(fid,'font-family: Georgia, "Times New Roman", Times, serif;\n');
    fprintf(fid,'font-size: 16px;\n');
    fprintf(fid,'}\n');
    fprintf(fid,'.style7 {\n');
    fprintf(fid,'font-family: Georgia, "Times New Roman", Times, serif;\n');
    fprintf(fid,'font-size: 14px;\n');
	fprintf(fid,'font-weight: bold;\n');
	fprintf(fid,'}\n');
	fprintf(fid,'.style9 {font-family: Georgia, "Times New Roman", Times, serif; font-weight: bold; }\n');
    fprintf(fid,'-->\n');
        fprintf(fid,'.style10 {\n');
    fprintf(fid,'font-family: Georgia, "Times New Roman", Times, serif;\n');
    fprintf(fid,'color: #990000;\n');
    fprintf(fid,'font-size: 12px;\n');
    fprintf(fid,'}\n');
    fprintf(fid,'.style11 {\n');
	fprintf(fid,'color: #0000CC;\n');
	fprintf(fid,'font-weight: bold;\n');
	fprintf(fid,'}\n');
    fprintf(fid,'-->\n');
    fprintf(fid,'</style>\n');
    
    fprintf(fid,'</HEAD> \n');
    fprintf(fid,'<BODY> \n');
    
    fprintf(fid,'<p class="style6"><strong>KAPattern result for the system</strong>:<em> ');
    fprintf(fid,'%s \n</em></p>', file);
    
    fprintf(fid,'<p class="style7"><a href="http://bbc.mcw.edu/Computation" target="_blank"> Computational Bioengineering Group</a> \n');
    fprintf(fid,'@ <a href="http://bbc.mcw.edu/" target="_blank"><img src="bbc.jpg" alt="BBC" width="208" height="38" /></a></p>\n');
    fprintf(fid,'<p class="style9">Medical College of Wisconsin');
    fprintf(fid,' <a href="http://www.mcw.edu/" target="_blank"><img src="MCW.jpg" alt="MCW" width="80" height="57" /></a></p>\n');
    
    % put the result here
    fprintf(fid, '<p class="style9">Here is the result: </p>');
    
    fprintf(fid,'<p><span class="style10">(If there is any problem to display the equations with IE, please download and install \n');
    fprintf(fid,'<span class="style11"><a href="http://www.dessci.com/en/products/mathplayer/download.htm?src=mplogo" \n');
    fprintf(fid,'target="_blank">MathPlayerSetup.exe</a></span> \n');
    fprintf(fid,'<a href="http://www.dessci.com/en/products/mathplayer/download.htm?src=mplogo">)</a> </span></p> \n');
    
    fprintf(fid, '<p class="style9">Enzyme forms: </p>');
    for ii = 1: length(state)
        str = char(state{ii});
        str = regexprep(str, ';', '');
        fprintf(fid, '<p> </p>\n');
        fprintf(fid, '<m:math>');
        
        index = find(str=='=');
        str1 = str(1:index);
        str2 = str(index+1:end);
        % part before '='
        for b = 1: length(str1)
            if (str(b) == 'E')
                strb = regexprep(str(b), 'E', '<m:mi>E</m:mi>');
                fprintf(fid,'%s\n',strb);
            else
                fprintf(fid,'<m:mn>');
                fprintf(fid,str(b));
                fprintf(fid,'</m:mn>\n');
            end
        end
        % dealing with part after '=' 
        index1 = find(str2=='+');
        TermNumber = length(index1) + 1;
        % seprated by '+'
        for i = 1 : TermNumber
            if(i > 1)
                fprintf(fid, '<m:mo>+</m:mo>');
            end
            if (i == 1)
                strtemp = strtrim(str2(1:index1(1)-1));
            elseif (i == TermNumber)
                strtemp = strtrim(str2(index1(end)+1: end));
            else
                strtemp = strtrim(str2(index1(i-1)+1:index1(i)-1));
            end
            % seperated by '*'
            index2 = find(strtemp == '*');
            SubTermNumber = length(index2) + 1;
            
            for j = 1 : SubTermNumber
                if(j == 1)
                    substrtemp = strtrim(strtemp(1: index2(1)-1));
                elseif (j == SubTermNumber)
                    substrtemp = strtrim(strtemp(index2(end)+1 : end));
                else
                    substrtemp = strtrim(strtemp(index2(j-1)+1 : index2(j)-1));
                end
                % first, check if it is 'k' term
                if( sum(substrtemp == 'k') > 0)
                    % check if it is '_' term
                    if( sum(substrtemp == '_') > 0 )                   
                        fprintf(fid, '<m:msub>');
                        fprintf(fid, '<m:mi>k</m:mi>');
                        fprintf(fid, '<m:mn>');
                        fprintf(fid, '-%s',substrtemp(3:end)); % with '-'
                        fprintf(fid, '</m:mn>');
                        fprintf(fid, '</m:msub>');
                    else
                        fprintf(fid, '<m:msub>');
                        fprintf(fid, '<m:mi>k</m:mi>');
                        fprintf(fid, '<m:mn>');
                        fprintf(fid, '%s',substrtemp(2:end));
                        fprintf(fid, '</m:mn>');
                        fprintf(fid, '</m:msub>');
                    end
                else
                    fprintf(fid,'<m:mi>');
                    fprintf(fid,'%s',substrtemp);
                    fprintf(fid,'</m:mi>');
                end
            end
        end
        fprintf(fid, '</m:math> \n');
    end    
    % LinkMatrix display
    fprintf(fid, '<p class="style9">Link Matrix: </p>\n');
    fprintf(fid, '<p> </p>\n');
    fprintf(fid, '<m:math> \n');
    fprintf(fid, '<m:matrix> \n'); 
    for i = 1 : size(LinkMatrix, 1)
        fprintf(fid, '<m:matrixrow> \n');
        for j = 1 : size(LinkMatrix, 2)
            fprintf(fid, '<m:cn> %i </m:cn> \n', LinkMatrix(i, j));
        end
        fprintf(fid, '</m:matrixrow> \n'); 
    end
    fprintf(fid, '</m:matrix> \n'); 
    fprintf(fid, '</m:math> \n');
    
    % KineticMatrix display
    fprintf(fid, '<p class="style9">Kinetic Matrix: </p>\n');
    fprintf(fid, '<p> </p>\n');
    fprintf(fid, '<m:math> \n');
    fprintf(fid, '<m:matrix> \n'); 
    for i = 1 : size(KineticMatrix, 1)
        fprintf(fid, '<m:matrixrow> \n');
        for j = 1 : size(LinkMatrix, 2)
            fprintf(fid, '<m:cn>');
            
            str = KineticMatrix{i, j};
            if (str == 0)
               fprintf(fid,'<m:mi>');
               fprintf(fid,'%s','0');
               fprintf(fid,'</m:mi>');
            else
                index3 = find(str == '*');
                TermNumber = length(index3) + 1;                
                if (TermNumber == 1) % only has 'k' term
                    if( sum(str == '_') > 0 )                   
                       fprintf(fid, '<m:msub>');
                       fprintf(fid, '<m:mi>k</m:mi>');
                       fprintf(fid, '<m:mn>');
                       fprintf(fid, '-%s',str(3:end)); % with '-'
                       fprintf(fid, '</m:mn>');
                       fprintf(fid, '</m:msub>');    
                    else
                        fprintf(fid, '<m:msub>');
                        fprintf(fid, '<m:mi>k</m:mi>');
                        fprintf(fid, '<m:mn>');
                        fprintf(fid, '%s',str(2:end));
                        fprintf(fid, '</m:mn>');
                        fprintf(fid, '</m:msub>');
                    end
                else % 'k' term and constant
                    index4 = find(str == '*');
                    SubTermNumber = length(index4) + 1;
            
                    for k = 1 : SubTermNumber
                        if(k == 1)
                            substr = str(1: index4(1)-1);
                        elseif (k == SubTermNumber)
                            substr = str(index4(end)+1 : end);
                        else
                            substr = str(index4(k-1)+1 : index2(k)-1);
                        end
                        % first, check if it is 'k' term
                        if( sum(substr == 'k') > 0)
                            % check if it is '_' term
                            if( sum(substr == '_') > 0 )                   
                                fprintf(fid, '<m:msub>');
                                fprintf(fid, '<m:mi>k</m:mi>');
                                fprintf(fid, '<m:mn>');
                                fprintf(fid, '-%s',substr(3:end)); % with '-'
                                fprintf(fid, '</m:mn>');
                                fprintf(fid, '</m:msub>');
                            else
                                fprintf(fid, '<m:msub>');
                                fprintf(fid, '<m:mi>k</m:mi>');
                                fprintf(fid, '<m:mn>');
                                fprintf(fid, '%s',substr(2:end));
                                fprintf(fid, '</m:mn>');
                                fprintf(fid, '</m:msub>');
                            end
                        else
                            fprintf(fid,'<m:mi>');
                            fprintf(fid,'%s',substr);
                            fprintf(fid,'</m:mi>');
                        end
                    end
                end                
            end
            fprintf(fid, '</m:cn>');
        end
        fprintf(fid, '</m:matrixrow> \n'); 
    end
    fprintf(fid, '</m:matrix> \n'); 
    fprintf(fid, '</m:math> \n');
    
    fprintf(fid, '<p class="style9">Reference: </p> \n');
    fprintf(fid, '<p><span class="style6">Feng Qi, Ranjan K. Dash, Yu Han, Daniel A. Beard, \n');
    fprintf(fid, '<em>"Generating Rate Equations for Complex Enzyme Systems by a Computer-Assisted Systematic Method."</em></span><strong><br /></strong></p> \n');
    fprintf(fid, '<p><span class="style3"> BMC Bioinformatics 2009, 10: 238 </p>\n');
    fprintf(fid,'</BODY></HTML>');
    fclose(fid);
    %winopen('ResultPage.html');
end