function [NodeList Link KineticMatrix] = ReadInput(FileName)

%--------------------------------------------------------------------------
% Function used to read the input file of the enzyme reaction system.
%
% USAGE:           
%            [NodeList Link KineticMatrix] = ReadInput(FileName)   
%
% INPUT:           
%            FileName  -- Input file name, no suffix
% 
% OUTPUT:          
%            NodeList  -- (Array) NodeList, i node
%
%                Link  -- (Cell Array)Link list, Link{i,3}, 
%                         i = 1 : NodeNumber,
%                         column 1: the number of the link for node i, 
%                         column 2: the list the the neighbor nodes number 
%                         for node i. 
%                         column 3: the list for the corresponding link 
%                         index number that connecting with node i.
%
%        KineticMatrix -- (Cell String) Stoichmetric matrix of the system, 
%                         element are the rate constants. 
%                         (pseudo-first rate constant)
%
% Note: the k_1 should be '_' not '-' in the input file.
%-----2008--MCW------------------------------------------------------------
%-- Ver. 1.2 -- F. Qi --
% Last updated: 2010-03-01

InputFileFlow = textread([FileName,'.txt'],'%s');
FileLength = length(InputFileFlow);
NodeList = [];

% build the KineticMatrix from input file
for i = 1 : FileLength/3     
    num1 = str2num(InputFileFlow{i*3-2});  %#ok<ST2NM>
    num2 = str2num(InputFileFlow{i*3-1});  %#ok<ST2NM>
    if(isempty(num1) || isempty(num2))
        errordlg(['No right information in input file!';' Please select another input file! '],'File Open Error');
        Link = {};
        KineticMatrix = {};
        return;
    else
    NodeList = [NodeList num1]; %#ok<AGROW>
    NodeList = [NodeList num2]; %#ok<AGROW>
    end
end

NodeList = unique(NodeList);

KineticMatrix{max(NodeList), max(NodeList)} = [];

for i = 1 : FileLength/3     
    num1 = str2num(InputFileFlow{i*3-2});  %#ok<ST2NM>
    num2 = str2num(InputFileFlow{i*3-1});  %#ok<ST2NM>
    if(isempty(KineticMatrix{num1,num2}))
        KineticMatrix{num1, num2} = InputFileFlow{i*3};
    else
        KineticMatrix{num1, num2} = [KineticMatrix{num1, num2},'+',InputFileFlow{i*3}]; 
    end
end

% set up the Link information
LinkMatrix = zeros(length(NodeList), length(NodeList));
Link = cell(size(KineticMatrix,1),3);
LinkIndex = 0;
for i = 1 : size(KineticMatrix,1)
    LinkNumber = 0;
    LinkList = [];
    for j = 1 : size(KineticMatrix,1)
        if(i == j || (isempty(KineticMatrix{i,j}) && isempty(KineticMatrix{j,i}) ) )  
            continue;
        else
            LinkNumber = LinkNumber + 1;
            LinkList = [LinkList j]; %#ok<AGROW>
            if(j>i) 
                LinkIndex = LinkIndex + 1;
                LinkMatrix(i,j) = LinkIndex;
                LinkMatrix(j,i) = LinkIndex;
            end
        end
    end
    Link(i,:) = {LinkNumber, LinkList, [LinkMatrix(i,LinkList(:))]}; %#ok<NBRAK>
end

% modify the KineticMatrix matrix, set [] to 0;
for i = 1 : size(KineticMatrix,1)
    for j = 1 : size(KineticMatrix,1)
        if(isempty(KineticMatrix{i,j}))
            KineticMatrix{i,j} = 0; %#ok<AGROW>
        end
    end
end

